## Backtrace-Overview

- Backtrace service to support the overview feature by serving and caching CRDB queries

![Database schema](db_schema.svg "Schema")

## Git hooks
- Please run `npm run prepare` to install Husky git hooks
  - Hooks scripts are available in .husky directory
  - pre-commit hook runs lint on any staged files
    - You can attempt to automatically fix lint errors with `npm run lint:fix`
  - pre-push hook runs npm test
  - temporarily disable husky by pre-pending `HUSKY=0` to any command

## Development
- `yarn watch-build` in one terminal
- `yarn watch-start` in a second terminal
- Alternatively: `NODE_ENV=development yarn watch-debug` in the second terminal to enable debugging via chrome devtools

### Developing with Postgres
- Postgres should be listening on localhost port 5432 (see overview_dev_pg.conf)
  - Create docker container for the first time: 
   `docker run --name postgres -p 5432:5432 -e POSTGRES_HOST_AUTH_METHOD=trust -e POSTGRES_DB=overviewdb -d postgres`
  - check status of docker container:
`docker ps`
`docker logs -f postgres`
- stop container:
`docker stop postgres`
- start container:
`docker start postgres`
- Begin interactive session:
- `psql -U postgres -d overviewdb -h localhost`
- Begin development
`yarn watch-build` as usual, and then `yarn watch-start-pg` in a second terminal

## Schema migrations
Create a new migration:
- `NODE_ENV=development yarn exec -- knex migrate:make add_position_to_repositories`

Migrations run automatically when the service starts up. 

*The below knex commands might not work any longer, due to some unresolevd typescript / knex incompatibility.*

Roll back one migration:
- `NODE_ENV=development yarn exec -- knex migrate:down`
More commands:
- `NODE_ENV=development yarn exec -- knex --help`
On a production server:
- `user@overview01:/opt/backtrace/services/backtrace-overview-code$ sudo NODE_ENV=production npm run-script exec -- ./node_modules/.bin/knex migrate:latest`

## Tasks
Task files can be used to write one-off scripts, such as data migrations.
See the `tasks/seed_defaults.ts` for an existing example.
