import { BacktraceService, Logger, PackageUtils } from 'backtrace-service';
import fs from 'fs';
import path from 'path';
import { getConfPath, isStandalone } from './src/lib/util';
import { IOverviewServiceConfig } from './src/main';

const log = Logger.getLogger();

const configDb = BacktraceService.readConfiguration<IOverviewServiceConfig>(
  'overview',
  6457,
  isStandalone ? getConfPath('overview') : undefined,
  true,
).database;

if (configDb.connection.filename) {
  // Check db directory if it does not already exist
  const directory = path.dirname(configDb.connection.filename);
  if (!fs.existsSync(directory)) {
    try {
      fs.mkdirSync(directory, { recursive: true });
    } catch (err) {
      log.error(`Error: Database directory ${directory} cannot be created.`);
      throw err;
    }
  }
}

// hacky, but we need to give /migrations when invoked from the knex CLI, and ./dist/migrations when invoked from the migration we do on service start
const migrationsDir = /knex/.test(process.env._ as string)
  ? path.join(PackageUtils.packageDir, './migrations')
  : path.join(PackageUtils.packageDir, './dist/migrations');

const defaults = {
  migrations: {
    directory: migrationsDir,
  },
  useNullAsDefault: true,
};

const conf = Object.assign(defaults, configDb);

export default conf;
